
From: Prasanna S Panchamukhi <prasanna@in.ibm.com>

Kprobes could not handle the insertion of a probe on the ret/lret
instruction and used to oops after single stepping since kprobes was
modifying eip/rip incorrectly.  Adjustment of eip/rip is not required after
single stepping in case of ret/lret instruction, because eip/rip points to
the correct location after execution of the ret/lret instruction.  This
patch fixes the above problem.

Signed-off-by: Prasanna S Panchamukhi <prasanna@in.ibm.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/arch/i386/kernel/kprobes.c   |    7 +++++++
 25-akpm/arch/x86_64/kernel/kprobes.c |    7 +++++++
 2 files changed, 14 insertions(+)

diff -puN arch/i386/kernel/kprobes.c~kprobes-incorrect-handling-of-probes-on-ret-lret-instruction arch/i386/kernel/kprobes.c
--- 25/arch/i386/kernel/kprobes.c~kprobes-incorrect-handling-of-probes-on-ret-lret-instruction	2005-04-01 00:49:09.000000000 -0800
+++ 25-akpm/arch/i386/kernel/kprobes.c	2005-04-01 00:49:09.000000000 -0800
@@ -217,6 +217,13 @@ static void resume_execution(struct kpro
 		*tos &= ~(TF_MASK | IF_MASK);
 		*tos |= kprobe_old_eflags;
 		break;
+	case 0xc3:		/* ret/lret */
+	case 0xcb:
+	case 0xc2:
+	case 0xca:
+		regs->eflags &= ~TF_MASK;
+		/* eip is already adjusted, no more changes required*/
+		return;
 	case 0xe8:		/* call relative - Fix return addr */
 		*tos = orig_eip + (*tos - copy_eip);
 		break;
diff -puN arch/x86_64/kernel/kprobes.c~kprobes-incorrect-handling-of-probes-on-ret-lret-instruction arch/x86_64/kernel/kprobes.c
--- 25/arch/x86_64/kernel/kprobes.c~kprobes-incorrect-handling-of-probes-on-ret-lret-instruction	2005-04-01 00:49:09.000000000 -0800
+++ 25-akpm/arch/x86_64/kernel/kprobes.c	2005-04-01 00:49:09.000000000 -0800
@@ -355,6 +355,13 @@ static void resume_execution(struct kpro
 		*tos &= ~(TF_MASK | IF_MASK);
 		*tos |= kprobe_old_rflags;
 		break;
+	case 0xc3:		/* ret/lret */
+	case 0xcb:
+	case 0xc2:
+	case 0xca:
+		regs->eflags &= ~TF_MASK;
+		/* rip is already adjusted, no more changes required*/
+		return;
 	case 0xe8:		/* call relative - Fix return addr */
 		*tos = orig_rip + (*tos - copy_rip);
 		break;
_
